% function r = admClean(baseDir, modes)
%
% Delete ADiMat-generated files of modes from the directory
% baseDir.
%
% baseDir is the name of a directory to cd to first. The default is
% '.', the current directory.
%
% modes is a string of 1-5 characters with the follwoing meaning:
%
%  - F: delete files generated by admDiffFor:                    g_*.m
%  - f: delete files generated by admDiffVFor:                   d_*.m
%  - r: delete files generated by admDiffRev and admTaylorRev:   a_*.m,  rec_*.m, ret_*.m
%  - t: delete files generated by admDiffTaylorFor:              t_*.m
%  - s: rmdir ADiMat status directory (silently):                .adimat
%
% The default value for modes is 'Ffrts'
%
% Be careful when using this function, since all files that fit into
% the above patterns are deleted.
%
% This file is part of the ADiMat runtime environment.
%
% Copyright 2013 Johannes Willkomm, Institute for Scientific Computing
%                     TU Darmstadt
function r = admClean(baseDir, modes)
  if nargin < 1
    baseDir = '.';
  end
  if nargin < 2
    modes = 'Ffrts';
  end
  
  if admIsOctave()
    confirm_recursive_rmdir(false, 'local')
  end

  res = false();
  
  curDir = pwd;
  
  cd(baseDir);
  
  res = true();
  for i=1:length(modes)
    m = modes(i);
    switch m
     case 'F'
      delete g_*.m
     case 'f'
      delete d_*.m
     case 'r'
      delete a_*.m
      delete rec_*.m
      delete ret_*.m
     case 't'
      delete t_*.m
     case 's'
      if exist('.adimat', 'dir')
        rmdir('.adimat', 's')
      end
    end
  end

  cd(curDir);

  if nargout > 0, r = res; end
  
% $Id: admClean.m 3898 2013-10-08 09:23:12Z willkomm $
